﻿/*
 * FILE:        KMeansClusteringController.cs
 *                                                                      
 * DESCRIPTION: The file contains implementation for providing the 'controller' functionality for K-Means clustering sample.
 *
 * VERSION:     v1.0, updated July 2011.
 *
 * Copyright © 2011 Microsoft Corporation. All rights reserved.
 */

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Microsoft.WindowsAzure.StorageClient;
using Research.MapReduce.Core;
using Research.MapReduce.Library;

namespace Research.MapReduce.Samples.KMeansClustering
{
    /// <summary>
    /// Provides the 'controller' functionality for K-Means clustering sample.
    /// </summary>
    [Controller(Name = "K-Means Clustering", Description = "K-Means clustering algorithm for CSV dataset that have atleast two numeric columns.")]
    public class KMeansClusteringController : Controller
    {
        #region Fields

        private const int SamplingSizeForDeterminingInitialClusters = 100;

        private BlobCsvPartitioner partitioner;
        private string[] inputDataHeader;
        private string[] clusteringColumnNames;
        private int firstClusterColumnIndex;
        private int secondClusterColumnIndex;
        private List<Point> newClusters;
        private List<Point> previousClusters;
        private int numberOfIterations;

        #endregion

        #region Input JobParameters

        /// <summary>
        /// The names of the columns to cluster on.
        /// </summary>        
        [ControllerParameter(Name = ParameterNames.ClusterColumns, Description = "The names of the columns (in CSV format) whose values are to be used as the dimensions of the cluster.", IsRequired = true)]
        public string ClusteringColumnNamesInCsv { get; set; }

        /// <summary>
        /// A place for the caller to provide the URI of the blob containing the initial centroids.
        /// The data should contain the value corresponding to the two cluster columns in CSV
        /// format.
        /// </summary>
        [ControllerParameter(Name = ParameterNames.InitialClustersLocation, Description = "Location of the initial centroids. The data should contain the value corresponding to the two cluster columns in CSV format.", IsRequired = false)]
        public string InitialClustersLocation { get; set; }

        /// <summary>
        /// The number of centroids desired, thus the number of total clusters.
        /// </summary>
        [ControllerParameter(Name = ParameterNames.Clusters, Description = "The number of clusters to find from the input data.", IsRequired = false, DefaultValue = "2")]
        public int NumberOfClusters { get; set; }

        /// <summary>
        /// Duration within which a single k-means job must complete.
        /// </summary>
        [ControllerParameter(Name = ParameterNames.JobTimeoutInMinutes, Description = "Duration within which a single k-means job must complete.", DefaultValue = "5", IsRequired = true)]
        public int JobTimeoutInMinutes { get; set; }

        #endregion

        #region Methods

        /// <summary>
        /// Entry point for the execution of <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringController"/>.
        /// </summary>
        public override void Run()
        {
            ValidateInputParameters();

            partitioner = new BlobCsvPartitioner(this.CloudClient, this.
                InputDataLocation, this.RequestedNumberOfMappers);
            inputDataHeader = partitioner.Header.Split(new char[] { ',' }, StringSplitOptions.
                RemoveEmptyEntries);
            firstClusterColumnIndex = inputDataHeader.GetIndexOf(clusteringColumnNames[0]);
            secondClusterColumnIndex = inputDataHeader.GetIndexOf(clusteringColumnNames[1]);

            Job kMeansJob = GetJob();

            if (string.IsNullOrEmpty(this.InitialClustersLocation) || string.IsNullOrWhiteSpace(
                this.InitialClustersLocation))
            {
                // If user has not provided initial set of clusters then randomly pick via
                // sampling the input dataset.
                previousClusters = GetInitialClustersViaInputSampling();
            }
            else
            {
                // Get the initial set of clusters from the parameter 'InitialClustersLocation'.
                previousClusters = GetInitialClustersFromInputParameter();
            }

            int count = 0;
            while (count < 20) // Maximum of 20 iterations. Otherwise, make it a while (true)
            {
                // Add the input clusters for use by mappers.
                kMeansJob.AddOrReplaceMapParameter(ParameterNames.InputClusters, previousClusters.
                    GetString());
                kMeansJob.GetReduceOutputsToController = true;

                kMeansJob.Run();

                numberOfIterations++;

                if (kMeansJob.JobExecutionInfo.State != JobState.Completed)
                {
                    // If the job failed, then return.
                    return;
                }

                // New clusters will be returned as reduce output.
                newClusters = new List<Point>();
                IEnumerable<IReduceResult<Point, WeightedPoint>> reduceOutput = kMeansJob.
                    GetReduceOutputs<Point, WeightedPoint>();
                foreach (KeyValuePair<Point, WeightedPoint> output in reduceOutput.SelectMany(
                    tuple => tuple))
                {
                    // Value is the new co-ordinates of cluster.                    
                    newClusters.Add(output.Value);
                }

                if (previousClusters.EqualsTo(newClusters))
                {
                    // If the newly computed clusters do not vary from the ones computed in the
                    // previous iteration, then no further computation is necessary.
                    break;
                }
                previousClusters = newClusters;
                count++;
            }

            // Write the clusters produced by different reducers into a single blob.
            PrepareFinalResult();
        }

        /// <summary>
        /// Validates the input parameters provided for running K-Means clustering.
        /// </summary>
        private void ValidateInputParameters()
        {
            if (string.IsNullOrEmpty(this.StorageConnectionString) || string.IsNullOrWhiteSpace(
                this.StorageConnectionString))
            {
                throw new ArgumentNullException("StorageConnectionString");
            }
            if (string.IsNullOrEmpty(this.InputDataLocation) || string.IsNullOrWhiteSpace(
                this.InputDataLocation))
            {
                throw new ArgumentNullException("InputDataLocation");
            }
            else
            {
                this.InputDataLocation.ValidateUriFormat("InputDataLocation");
                ValidateInputLocation(this.InputDataLocation, this.CloudClient);
            }

            if (string.IsNullOrEmpty(this.OutputDataLocation) || string.IsNullOrWhiteSpace(
                this.OutputDataLocation))
            {
                throw new ArgumentNullException("OutputDataLocation");
            }
            else
            {
                try
                {
                    CloudBlobContainer outputContainer = this.CloudClient.BlobClient.GetContainerReference(
                        OutputDataLocation);
                    outputContainer.CreateIfNotExist();
                    outputContainer.Clean();
                }
                catch (StorageException ex)
                {
                    throw new ArgumentException(
                        string.Format(
                            "Error occurred while trying to initialize output location [{0}]",
                            this.OutputDataLocation),
                        ex);
                }
            }

            if (this.RequestedNumberOfMappers <= 0)
            {
                throw new ArgumentException("Mappers should be greater than zero.");
            }

            if (this.RequestedNumberOfReducers <= 0)
            {
                throw new ArgumentException("Reducers should be greater than zero.");
            }

            if (string.IsNullOrEmpty(this.ClusteringColumnNamesInCsv) || string.IsNullOrWhiteSpace(
                this.ClusteringColumnNamesInCsv))
            {
                throw new ArgumentNullException("ClusterColumns");
            }
            else
            {
                clusteringColumnNames = this.ClusteringColumnNamesInCsv.Split(',');
                if (clusteringColumnNames.Length != 2)
                {
                    throw new ArgumentException("Only two clustering columns can be used.");
                }
            }

            if (this.JobTimeoutInMinutes <= 0)
            {
                throw new ArgumentException("JobTimeoutInMinutes should be greater than zero.");
            }

            if (!string.IsNullOrEmpty(this.InitialClustersLocation) && !string.IsNullOrWhiteSpace(
                this.InitialClustersLocation))
            {
                this.InitialClustersLocation.ValidateUriFormat("InitialClustersLocation");
                ValidateInputLocation(this.InitialClustersLocation, this.CloudClient);
            }
            else if (this.NumberOfClusters <= 0)
            {
                throw new ArgumentException("Clusters should be greater than zero.");
            }
        }

        /// <summary>
        /// Instantiates and returns <see cref="Research.MapReduce.Core.Job" /> for running K-Means
        /// clustering using the specified values for input parameters.
        /// </summary>
        /// <returns>An instance of <see cref="Research.MapReduce.Core.Job" />.</returns>
        private Job GetJob()
        {
            JobConfiguration jobConfig = new JobConfiguration();
            jobConfig.MapperType = typeof(KMeansClusteringMapper);
            jobConfig.CombinerType = typeof(KMeansClusteringReducer);
            jobConfig.MapOutputStorage = MapOutputStoreType.Local;
            jobConfig.KeyPartitioner = typeof(HashModuloKeyPartitioner<Point, WeightedPoint>);
            jobConfig.ReducerType = typeof(KMeansClusteringReducer);
            jobConfig.JobTimeout = TimeSpan.FromMinutes(JobTimeoutInMinutes);
            jobConfig.ExceptionPolicy = new TerminateOnFirstException();
            jobConfig.JobParameters.Add(ParameterNames.FirstClusterColumnIndex, firstClusterColumnIndex
                .ToString());
            jobConfig.JobParameters.Add(ParameterNames.SecondClusterColumnIndex, secondClusterColumnIndex
                .ToString());

            // Use simple iterative task scheduling policy.
            jobConfig.TaskSchedulingPolicy = TaskSchedulingPolicy.SimpleIterative;

            Job job = new Job(jobConfig, this);
            job.DataPartitioner = partitioner;
            job.NoOfReduceTasks = this.RequestedNumberOfReducers;
            job.RecordWriter = new KMeansClusteringWriter(this.CloudClient, this.OutputDataLocation);

            // Use data cache if available.
            job.UseCacheIfAvailable = true;
            return job;
        }

        /// <summary>
        /// Randomly picks initial clusters by sampling the input dataset.
        /// </summary>
        /// <returns>A list of <see cref="Research.MapReduce.Samples.KMeansClustering.Point" /> 
        /// which will be treated as initial clusters.</returns>
        private List<Point> GetInitialClustersViaInputSampling()
        {
            List<Point> initialClusters = new List<Point>();
            List<string> samples = PickRandomRecordsFromInput();
            Random random = new Random();

            for (int i = 0; i < this.NumberOfClusters; i++)
            {
                int randomSampleNo = random.Next(SamplingSizeForDeterminingInitialClusters);
                string[] randomSample = samples[randomSampleNo].Split(new char[] { ',' },
                    StringSplitOptions.RemoveEmptyEntries);
                initialClusters.Add(new Point(randomSample[firstClusterColumnIndex],
                    randomSample[secondClusterColumnIndex]));
            }
            return initialClusters;
        }

        /// <summary>
        /// Picks random records from the input data for generating initial clusters. The number of
        /// records to be picked is specified by the constant 'SamplingSizeForDeterminingInitialClusters'.
        /// </summary>
        /// <returns>List of string containing the random records.</returns>
        private List<string> PickRandomRecordsFromInput()
        {
            int counter = 0;
            List<string> samples = new List<string>();
            BlobCsvPartitioner dataPartitioner = new BlobCsvPartitioner(this.CloudClient, this.
                InputDataLocation, this.RequestedNumberOfMappers);
            Random rand = new Random();

            foreach (IDataPartition partition in dataPartitioner.GetPartitions())
            {
                IRecordReader<int, string> reader = dataPartitioner.GetRecordReader(partition);
                foreach (KeyValuePair<int, string> record in reader.GetRecords())
                {
                    samples.Add(record.Value);
                    if (++counter == SamplingSizeForDeterminingInitialClusters)
                    {
                        return samples;
                    }
                }
            }
            return samples;
        }

        /// <summary>
        /// Gets the initial set of clusters from the parameter 'InitialClustersLocation'.
        /// </summary>
        /// <returns>List of <see cref="Research.MapReduce.Samples.KMeansClustering.Point" /> 
        /// which represent the initial clusters.</returns>
        private List<Point> GetInitialClustersFromInputParameter()
        {
            List<Point> centroids = new List<Point>();
            CloudBlob centroidsBlob = this.CloudClient.BlobClient.GetBlobReference(this.
                InitialClustersLocation);
            using (StreamReader sr = new StreamReader(centroidsBlob.OpenRead()))
            {
                string cluster = string.Empty;
                while ((cluster = sr.ReadLine()) != null)
                {
                    try
                    {
                        centroids.Add(new Point(cluster));
                    }
                    catch (FormatException ex)
                    {
                        throw new ArgumentException(
                            string.Format(
                                "Error occurred while processing the initial cluster: [0]",
                                cluster), ex);
                    }
                    catch (ArgumentNullException ex)
                    {
                        throw new ArgumentException(
                            string.Format(
                                "Error occurred while processing the initial cluster: [0]",
                                cluster), ex);
                    }
                    catch (OverflowException ex)
                    {
                        throw new ArgumentException(
                            string.Format(
                                "Error occurred while processing the initial cluster: [0]",
                                cluster), ex);
                    }
                }
            }
            return centroids;
        }

        /// <summary>
        /// Write the clusters produced by different reducers to a single blob.
        /// </summary>                
        private void PrepareFinalResult()
        {
            CloudBlobContainer outputContainer = this.CloudClient.BlobClient.GetContainerReference(
                this.OutputDataLocation);
            CloudBlob finalClustersBlob = outputContainer.GetBlobReference("finalclusters");
            using (StreamWriter sw = new StreamWriter(finalClustersBlob.OpenWrite()))
            {
                foreach (Point centroid in newClusters)
                {
                    sw.WriteLine("Cluster-{0},{1}", Guid.NewGuid(), centroid);
                }
            }
            this.Results.Add(ResultNames.FinalClustersBlobUri, finalClustersBlob.Uri.AbsoluteUri);
            this.Results.Add(ResultNames.NumberOfIterations, numberOfIterations.ToString());
        }

        /// <summary>
        /// Validates that the specified <see cref="Microsoft.WindowsAzure.StorageClient.CloudBlob" /> 
        /// exists and is not empty.
        /// </summary>
        /// <param name="location">Name of the  <see cref="Microsoft.WindowsAzure.StorageClient.
        /// CloudBlob" /> to be validated.</param>
        /// <param name="cloudClient">Instance of the <see cref="Research.MapReduce.Core.
        /// CloudClient" /></param>
        private static void ValidateInputLocation(string location, CloudClient cloudClient)
        {
            try
            {
                CloudBlob blob = cloudClient.BlobClient.GetBlobReference(location);
                blob.FetchAttributes();
                if (blob.Properties.Length <= 0)
                {
                    throw new ArgumentException(
                        string.Format(
                            "Input location [{0}] is empty.",
                            location));
                }
            }
            catch (StorageException ex)
            {
                throw new ArgumentException(
                    string.Format(
                        "Unable to access input location [{0}].",
                        location),
                    ex);
            }
        }

        #endregion
    }
}
