﻿/*
 * FILE:        KMeansClusteringMapper.cs
 *                                                                      
 * DESCRIPTION: The file contains implementation for providing map functionality for K-Means clustering sample by computing the nearest cluster for the input record.
 *
 * VERSION:     v1.0, updated July 2011.
 *
 * Copyright © 2011 Microsoft Corporation. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Research.MapReduce.Core;

namespace Research.MapReduce.Samples.KMeansClustering
{
    /// <summary>
    /// Provides the map functionality for K-Means clustering sample by computing the nearest cluster
    /// for the input record.
    /// </summary>
    public class KMeansClusteringMapper : IMapper<int, string, Point, WeightedPoint>
    {
        #region Fields

        private List<Point> inputClusters;
        private int firstClusterColumnIndex;
        private int secondClusterColumnIndex;

        #endregion

        #region Methods

        /// <summary>
        /// Configures the <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringMapper"/> 
        /// so as to retrieve input clusters and index of cluster columns in the input data.
        /// </summary>
        /// <param name="context">Instance of the <see cref="Research.MapReduce.Core.MapContext{K, V}" />.
        /// </param>
        public void Configure(MapContext<int, string> context)
        {
            inputClusters = context.TaskParameters[ParameterNames.InputClusters].ToString().
                GetPoints();
            firstClusterColumnIndex = int.Parse(context.JobParameters[ParameterNames.
                FirstClusterColumnIndex].ToString());
            secondClusterColumnIndex = int.Parse(context.JobParameters[ParameterNames.
                SecondClusterColumnIndex].ToString());
        }

        /// <summary>
        /// Computes the nearest cluster for for the input record.
        /// </summary>    
        /// <param name="key">Key of the record.</param>
        /// <param name="value">String representation of the input <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />.
        /// </param>
        /// <param name="context">Instance of the <see cref="Research.MapReduce.Core.MapContext{K, V}" />.
        /// </param>
        /// <returns>An enumerable of (nearestClusterPoint, <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint" />.) 
        /// key, value pairs.</returns>
        public IEnumerable<KeyValuePair<Point, WeightedPoint>> Map(int key, string value, MapContext<int, string> context)
        {
            Point point = value.GetPoint(firstClusterColumnIndex, secondClusterColumnIndex);
            Point nearestCluster = point.GetNearestPoint(inputClusters);
            yield return new KeyValuePair<Point, WeightedPoint>(nearestCluster, point.
                GetWeightedPoint());
        }

        #endregion
    }
}
