﻿/*
 * FILE:        KMeansClusteringReducer.cs
 *                                                                      
 * DESCRIPTION: The file contains implementation for providing combine and reduce functionality for K-Means clustering sample.
 *
 * VERSION:     v1.0, updated July 2011.
 *
 * Copyright © 2011 Microsoft Corporation. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Research.MapReduce.Core;

namespace Research.MapReduce.Samples.KMeansClustering
{
    /// <summary>
    /// Provides the combine and reduce functionality for K-Means clustering alogirthm. 
    /// </summary>
    /// <remarks>
    /// When acting as combiner, it returns the weighted sum of all points that have the same
    /// 'nearest cluster. When acting as reducer, it returns the average of all points that have
    /// the same 'nearest cluster'.
    /// </remarks>
    public class KMeansClusteringReducer : IReducer<Point, WeightedPoint, Point, WeightedPoint>
    {
        #region Methods

        /// <summary>
        /// Performs combine or reduce for K-Means clustering algorithm.
        /// </summary>        
        /// <param name="key">The <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />
        /// representing a cluster.</param>
        /// <param name="values">An enumerable of <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint" /> 
        /// representing the records for which the specified cluster(key) is nearest.</param>        
        /// <param name="context">Instance of <see cref="Research.MapReduce.Core.ReduceContext{K, V}" /> </param>
        /// <returns>Enumerable of (<see cref="Research.MapReduce.Samples.KMeansClustering.Point" /> ,
        /// <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint" /> ) key, value pairs 
        /// after performing combine or reduce operation.</returns>
        public IEnumerable<KeyValuePair<Point, WeightedPoint>> Reduce(Point key, IEnumerable<WeightedPoint> values, ReduceContext<Point, WeightedPoint> context)
        {
            WeightedPoint valuesSum = new WeightedPoint();
            foreach (WeightedPoint val in values)
            {
                valuesSum.Add(val);
            }
            if (context.CreatedForCombining)
            {
                yield return new KeyValuePair<Point, WeightedPoint>(key, valuesSum);
            }
            else
            {
                yield return new KeyValuePair<Point, WeightedPoint>(key, valuesSum.Average());
            }
        }

        /// <summary>
        /// Empty method implementation since there is nothing to configure.
        /// </summary>
        /// <param name="context">Instance of <see cref="Research.MapReduce.Core.ReduceContext{K, V}" /></param>
        public void Configure(ReduceContext<Point, WeightedPoint> context)
        { }

        #endregion
    }
}
