﻿/*
 * FILE:        KMeansClusteringResult.cs
 *                                                                      
 * DESCRIPTION: The file contains implementation for representing the result produced by 'KMeansClusteringReducer'
 *
 * VERSION:     v1.0, updated July 2011.
 *
 * Copyright © 2011 Microsoft Corporation. All rights reserved.
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using Microsoft.WindowsAzure.StorageClient;
using Research.MapReduce.Core;

namespace Research.MapReduce.Samples.KMeansClustering
{
    /// <summary>
    /// Represents the result produced by <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringReducer"/>. 
    /// </summary>
    [Serializable]
    public class KMeansClusteringResult : IReduceResult<Point, WeightedPoint>
    {
        #region Properties

        /// <summary>
        /// Buffer to store the result if the size is less than 4MB.
        /// </summary>
        public byte[] Buffer { get; private set; }

        /// <summary>
        /// Uri of the < containing the result if the size is greater than 4MB.
        /// </summary>
        public string BlobUri { get; private set; }

        /// <summary>
        /// Instance of <see cref="Research.MapReduce.Core.CloudClient"/> for accessing azure
        /// blob storage.
        /// </summary>
        public CloudClient CloudClient { get; private set; }

        #endregion

        #region Constructors

        /// <summary>
        /// Creates an instance of <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringResult"/>.
        /// </summary>
        /// <param name="cloudClient">Instance of <see cref="Research.MapReduce.Core.CloudClient"/>
        /// for accessing azure blob storage.</param>
        /// <param name="blobUri">Uri of the azure blob containing the output produced by 
        /// <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringReducer"/>.
        /// </param>
        public KMeansClusteringResult(CloudClient cloudClient, string blobUri)
        {
            if (cloudClient == null)
            {
                throw new ArgumentNullException("cloudClient");
            }
            if (string.IsNullOrEmpty(blobUri) || string.IsNullOrWhiteSpace(blobUri))
            {
                throw new ArgumentNullException("blobUri");
            }
            if (!Uri.IsWellFormedUriString(blobUri, UriKind.Absolute))
            {
                throw new ArgumentException("Invalid uri format.", "blobUri");
            }
            this.CloudClient = cloudClient;
            this.BlobUri = blobUri;
        }

        /// <summary>
        /// Creates an instance of <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringResult"/>.
        /// </summary>
        /// <param name="cloudClient">Instance of <see cref="Research.MapReduce.Core.CloudClient"/>
        /// for accessing azure blob storage.</param>
        /// <param name="buffer">The buffer containing the output produced by 
        /// <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringReducer"/>.
        /// </param>
        public KMeansClusteringResult(CloudClient cloudClient, byte[] buffer)
        {
            if (cloudClient == null)
            {
                throw new ArgumentNullException("cloudClient");
            }
            if (buffer == null)
            {
                throw new ArgumentNullException("buffer");
            }
            this.CloudClient = cloudClient;
            this.Buffer = buffer;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets an instance of <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringResult" /> 
        /// for iterating over the output of the KMeansClustering.
        /// </summary>
        /// <returns>A <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringResult" /> 
        /// enumerator.</returns>
        public IEnumerator<KeyValuePair<Point, WeightedPoint>> GetEnumerator()
        {
            return new KMeansClusteringResultEnumerator(this);
        }

        /// <summary>
        /// Gets an instance of <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringResult" /> 
        /// for iterating over the output of the KMeansClustering.
        /// </summary>     
        /// <returns>A <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringResult" /> 
        /// enumerator.</returns>
        IEnumerator IEnumerable.GetEnumerator()
        {
            return new KMeansClusteringResultEnumerator(this);
        }

        #endregion

        #region Nested Types

        /// <summary>
        /// Provides an enumerator for iterating through results produced by 
        /// <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringWriter"/>.
        /// </summary>
        private class KMeansClusteringResultEnumerator : IEnumerator<KeyValuePair<Point, WeightedPoint>>
        {
            private StreamReader reader;
            private KeyValuePair<Point, WeightedPoint> current;

            public KMeansClusteringResultEnumerator(KMeansClusteringResult result)
            {
                if (result.Buffer != null)
                {
                    reader = new StreamReader(new MemoryStream(result.Buffer));
                }
                else
                {
                    CloudBlob blob = result.CloudClient.BlobClient.GetBlobReference(result.
                        BlobUri);
                    reader = new StreamReader(blob.OpenRead());
                }
            }

            #region IEnumerator implementation

            /// <summary>
            /// Gets the current element of the collection.
            /// </summary>
            public KeyValuePair<Point, WeightedPoint> Current
            {
                get { return current; }
            }

            /// <summary>
            /// Releases resources used by the reader.
            /// </summary>
            public void Dispose()
            {
                if (reader != null)
                {
                    reader.Dispose();
                }
            }

            /// <summary>
            /// Gets the current element of the collection.
            /// </summary>
            object IEnumerator.Current
            {
                get { return current; }
            }

            /// <summary>
            /// Returns True if reader has not reached end of stream. Returns false otherwise.
            /// </summary>
            /// <returns>A boolean value to indicate whether the reader has reached end of stream 
            /// or not.</returns>
            public bool MoveNext()
            {
                if (reader.EndOfStream)
                {
                    return false;
                }
                string[] record = reader.ReadLine().Split('#');
                current = new KeyValuePair<Point, WeightedPoint>(new Point(record[0]), new
                    WeightedPoint(record[1]));
                return true;
            }

            /// <summary>
            /// Resets the enumerator.(Not supported)
            /// </summary>
            public void Reset()
            {
                throw new NotSupportedException();
            }
            #endregion
        }

        #endregion
    }
}
