﻿/*
 * FILE:        KMeansClusteringWriter.cs
 *                                                                      
 * DESCRIPTION: The file contains implementation for writing the output produced by 'KMeansClusteringReducer'.
 *
 * VERSION:     v1.0, updated July 2011.
 *
 * Copyright © 2011 Microsoft Corporation. All rights reserved.
 */

using System;
using System.Collections.Generic;
using System.IO;
using Microsoft.WindowsAzure.StorageClient;
using Research.MapReduce.Core;

namespace Research.MapReduce.Samples.KMeansClustering
{
    /// <summary>
    /// Writes the output produced by <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringReducer"/>.
    /// </summary>
    /// <remarks>
    /// The output is written to an internal buffer if size is less than 4MB, otherwise its written
    /// to windows azure blob storage.
    /// </remarks>
    [Serializable]
    public class KMeansClusteringWriter : IRecordWriter<Point, WeightedPoint>
    {
        #region Fields

        [NonSerialized]
        private const int BufferSize = 4 * 1024 * 1024; // 4MB
        [NonSerialized]
        private bool bufferFull;
        [NonSerialized]
        protected CloudBlob blob;
        [NonSerialized]
        private byte[] buffer;
        [NonSerialized]
        private int noOfBytesWrittenInBuffer;

        #endregion

        #region Properties

        /// <summary>
        /// Instance of <see cref="CloudClient"/> for accessing azure blob service client.
        /// </summary>        
        public CloudClient CloudClient { get; private set; }

        /// <summary>
        /// The blob output container in which a new blob will be created for writing the output.
        /// </summary>
        public string ContainerName { get; private set; }

        #endregion

        #region Constructors

        /// <summary>
        /// Creates an instance of <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringWriter"/>.
        /// </summary>
        /// <param name="cloudClient">Instance of <see cref="Research.MapReduce.Core.CloudClient"/>
        /// for accessing azure blob service client.</param>
        /// <param name="containerName">Name of the azure blob container to which output of
        /// <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringReducer"/>
        /// is to be written if it exceeds the buffer size (4MB).</param>
        public KMeansClusteringWriter(CloudClient cloudClient, string containerName)
        {
            if (cloudClient == null)
            {
                throw new ArgumentNullException("cloudClient");
            }
            if (string.IsNullOrEmpty(containerName) || string.IsNullOrWhiteSpace(containerName))
            {
                throw new ArgumentException("containerName");
            }

            this.CloudClient = cloudClient;
            this.ContainerName = containerName;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Writes the records to an <see cref="Microsoft.WindowsAzure.StorageClient.CloudBlob" /> 
        /// specified by 'outputPartition' if buffer size (4MB) has exceeded.
        /// </summary>
        /// <param name="outputPartition">Name of <see cref="Microsoft.WindowsAzure.StorageClient.CloudBlob" /> 
        /// to which the reduce output will be written.</param>
        /// <param name="records">The output produced by the <see cref="Research.MapReduce.Samples.
        /// KMeansClustering.KMeansClusteringReducer" />.</param>
        public void Write(string outputPartition, IEnumerable<KeyValuePair<Point, WeightedPoint>> records)
        {
            if (string.IsNullOrEmpty(outputPartition) || string.IsNullOrWhiteSpace(outputPartition))
            {
                throw new ArgumentException("outputPartition");
            }
            if (records == null)
            {
                throw new ArgumentNullException("records");
            }

            CloudBlobContainer container = this.CloudClient.BlobClient.GetContainerReference(this.
                ContainerName);
            container.CreateIfNotExist();
            blob = container.GetBlobReference(outputPartition);
            buffer = new byte[BufferSize];
            IEnumerator<KeyValuePair<Point, WeightedPoint>> enumerator = records.GetEnumerator();
            WriteToBuffer(enumerator, buffer);
            using (Stream stream = blob.OpenWrite())
            {
                stream.Write(buffer, 0, noOfBytesWrittenInBuffer);
                if (bufferFull)
                {
                    using (StreamWriter sw = new StreamWriter(stream))
                    {
                        // Re-write the current record as the buffer is full.
                        WriteRecord(sw, enumerator.Current);
                        while (enumerator.MoveNext())
                        {
                            WriteRecord(sw, enumerator.Current);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Gets the result containing the output of <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringReducer"/>.
        /// </summary>        
        /// <returns>An instance of the <see cref="Research.MapReduce.Samples.KMeansClustering.KMeansClusteringResult" />.
        /// </returns>
        public IReduceResult<Point, WeightedPoint> GetResult()
        {
            if (bufferFull)
            {
                return new KMeansClusteringResult(this.CloudClient, blob.Uri.AbsoluteUri);
            }
            else
            {
                byte[] localBuffer = new byte[noOfBytesWrittenInBuffer];
                Buffer.BlockCopy(buffer, 0, localBuffer, 0, noOfBytesWrittenInBuffer);
                return new KMeansClusteringResult(this.CloudClient, localBuffer);
            }
        }

        /// <summary>
        /// Writes all elements of the enumerator to the buffer.
        /// </summary>        
        /// <param name="enumerator">The enumerator to iterate over the key-value pairs
        /// (<see cref="Research.MapReduce.Samples.KMeansClustering.Point" />, <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint" />).
        /// </param>
        /// <param name="buffer">The byte array to which the elements are to be written.</param>
        private void WriteToBuffer(IEnumerator<KeyValuePair<Point, WeightedPoint>> enumerator, byte[] buffer)
        {
            try
            {
                using (MemoryStream ms = new MemoryStream(buffer))
                {
                    using (StreamWriter sw = new StreamWriter(ms))
                    {
                        while (enumerator.MoveNext())
                        {
                            //Write the current element in a specific # separated format.
                            WriteRecord(sw, enumerator.Current);
                            sw.Flush();
                            noOfBytesWrittenInBuffer = (int)ms.Position;
                        }
                    }
                }
            }
            catch (NotSupportedException)
            {
                bufferFull = true;
            }
        }

        /// <summary>
        /// Writes the key-value pair (<see cref="Research.MapReduce.Samples.KMeansClustering.Point" />,<see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint" />) 
        /// as strings seperated by '#' onto a new line.
        /// </summary>        
        /// <param name="sw">The <see cref="System.IO.StreamWriter"/> being used for wrigin the
        /// key-value pair.</param>
        /// <param name="record">The key-value pair.</param>
        private void WriteRecord(StreamWriter sw, KeyValuePair<Point, WeightedPoint> record)
        {
            sw.Write(record.Key);
            sw.Write("#");
            sw.Write(record.Value);
            sw.WriteLine();
        }

        #endregion
    }
}
