﻿/*
 * FILE:        MethodExtensions.cs
 *                                                                      
 * DESCRIPTION: The file contains implementation for common purpose extension methods.
 *
 * VERSION:     v1.0, updated July 2011.
 *
 * Copyright © 2011 Microsoft Corporation. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.WindowsAzure.StorageClient;
using System.Text;
using System.Linq;

namespace Research.MapReduce.Samples.KMeansClustering
{
    /// <summary>
    /// Contains common purpose extension methods.
    /// </summary>
    internal static class MethodExtensions
    {
        #region Methods

        /// <summary>
        /// From a given list of <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />, 
        /// determines the one nearest to the specified 'inputPoint'.
        /// </summary>
        /// <param name="inputPoint">Input <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />
        /// for which the nearest point from list is to be retrieved.</param>
        /// <param name="points">List of <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />s,  
        /// from which nearest point is to be determined.</param>
        /// <returns>The nearest <see cref="Research.MapReduce.Samples.KMeansClustering.Point" /> to the 
        /// 'inputPoint'.</returns>
        public static Point GetNearestPoint(this Point inputPoint, IEnumerable<Point> points)
        {
            double minValue = double.MaxValue;
            double diff = 0;
            Point nearestCentroid = null;

            foreach (Point centroid in points)
            {
                //Calculate the distance between 'point' and 'centroid'.
                diff = Math.Sqrt(Math.Pow(inputPoint.X - centroid.X, 2) + Math.Pow(inputPoint.Y - centroid.Y, 2));
                if (diff < minValue)
                {
                    minValue = diff;
                    nearestCentroid = centroid;
                }
            }
            return nearestCentroid;
        }

        /// <summary>
        /// Gets a <see cref="Research.MapReduce.Samples.KMeansClustering.Point" /> object from a string
        /// representation of the point.
        /// </summary>
        /// <param name="val"><see cref="Research.MapReduce.Samples.KMeansClustering.Point" /> in the 
        /// string format.</param>
        /// <param name="indexX">Index of X co-ordinate in the string.</param>
        /// <param name="indexY">Index of Y co-ordinate in the string.</param>
        /// <returns>An instance of <see cref="Research.MapReduce.Samples.KMeansClustering.Point" /> 
        /// converted from the string representation of that point.</returns>
        public static Point GetPoint(this string val, int indexX, int indexY)
        {
            string[] splitVal = val.Split(',');
            return new Point(splitVal[indexX], splitVal[indexY]);
        }

        /// <summary>
        /// Converts an instance of a <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />
        /// to a <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint" />, with weight as 1.
        /// </summary>
        /// <param name="point">Point to be converted to WeightedPoint.</param>
        /// <returns>The WeightedPoint object.</returns>
        public static WeightedPoint GetWeightedPoint(this Point point)
        {
            return new WeightedPoint(point.X, point.Y, 1);
        }

        /// <summary>
        /// Gets the index of a string in a list of strings. Returns -1 if the list of strings 
        /// does not contain the searched string.
        /// </summary>
        /// <param name="vals">List of strings.</param>
        /// <param name="val">String to be searched.</param>
        /// <returns>Index of input string in the list of strings.</returns>
        public static int GetIndexOf(this string[] vals, string val)
        {
            for (int i = 0; i < vals.Length; i++)
            {
                if (string.Equals(val, vals[i].Trim(), StringComparison.OrdinalIgnoreCase))
                {
                    return i;
                }
            }
            return -1;
        }
        
        /// <summary>
        /// Deletes all <see cref="Microsoft.WindowsAzure.StorageClient.CloudBlob" /> in the 
        /// specified <see cref="Microsoft.WindowsAzure.StorageClient.CloudBlobContainer" />.
        /// </summary>
        /// <param name="container">The <see cref="Microsoft.WindowsAzure.StorageClient.
        /// CloudBlobContainer" /> to be cleaned.</param>
        public static void Clean(this CloudBlobContainer container)
        {
            foreach (CloudBlob blob in container.ListBlobs(new BlobRequestOptions
            {
                UseFlatBlobListing = true
            }))
            {
                blob.Delete();
            }
        }

        /// <summary>
        /// Validates that the string is a well-formed uri. Throws an exception if the input
        /// string is not a well-formed uri.
        /// </summary>
        /// <param name="uri">String to be validated.</param>
        /// <param name="uriParamName">Name of the parameter.</param>
        /// <exception cref="System.ArgumentException"/>
        public static void ValidateUriFormat(this string uri, string uriParamName)
        {
            if (!Uri.IsWellFormedUriString(uri, UriKind.Absolute))
            {
                throw new ArgumentException(
                    string.Format(
                        "Value [{0}] specified for [{1}] is not a valid uri.",
                        uri,
                        uriParamName));
            }
        }

        /// <summary>
        /// Converts all <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />s in the list of 
        /// points to a single string.(Points are seperated from each other by a #.)
        /// </summary>
        /// <param name="points">List of points to be converted to a single string.</param>
        /// <returns>A single concatenated string representing the input points.(Points are 
        /// seperated from each other by a #.)</returns>
        public static string GetString(this List<Point> points)
        {
            StringBuilder pointsStr = new StringBuilder();
            foreach (Point point in points)
            {
                pointsStr.AppendFormat("{0}#", point.ToString());
            }
            return pointsStr.ToString().Trim('#');
        }

        /// <summary>
        /// Converts the <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />s in a single 
        /// string(Each point represented as CSV and '#' separator between points) to a list of points.
        /// </summary>
        /// <param name="pointsStr">String containing all points.</param>
        /// <returns>List of points represented by the input string.</returns>
        public static List<Point> GetPoints(this string pointsStr)
        {
            List<Point> points = new List<Point>();
            foreach (string point in pointsStr.Split('#'))
            {
                points.Add(new Point(point));
            }
            return points;
        }

        /// <summary>
        /// Converts a string array to a single string by concatenating them with a space between
        /// them.
        /// </summary>
        /// <param name="strArray">Array containing the strings that need to be concatenated.</param>
        /// <returns>Concatenated string representing the input string array.</returns>
        public static string GetString(this string[] strArray)
        {
            StringBuilder builder = new StringBuilder();
            foreach (string str in strArray)
            {
                builder.AppendFormat("{0} ", str);
            }
            return builder.ToString().TrimEnd();
        }

        /// <summary>
        /// Checks for equality of the current list of <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />
        /// against another list of <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />.
        /// </summary>
        /// <param name="inputPoints">Current list of <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />.
        /// </param>
        /// <param name="pointsToCompare">The list of <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />
        /// against which 'inputPoints' is to be compared.</param>
        /// <returns></returns>
        public static bool EqualsTo(this List<Point> inputPoints, List<Point> pointsToCompare)
        {
            if (inputPoints.Count != pointsToCompare.Count)
            {
                return false;
            }
            foreach (Point inputPoint in inputPoints)
            {
                if (!pointsToCompare.Contains(inputPoint, new PointEqualityComparer()))
                {
                    return false;
                }
            }

            return true;
        }

        #endregion
    }
}
