﻿/*
 * FILE:        Point.cs
 *                                                                      
 * DESCRIPTION: The file contains implementation for representing the cluster columns in a numerical format.
 *
 * VERSION:     v1.0, updated July 2011.
 *
 * Copyright © 2011 Microsoft Corporation. All rights reserved.
 */

using System;
using System.IO;
using Research.MapReduce.Core;
namespace Research.MapReduce.Samples.KMeansClustering
{
    /// <summary>
    /// Represents the cluster columns in a numerical format (by converting column values into 
    /// co-ordinates of a 2D point).
    /// </summary>
    public class Point : IBinarySerializable, IComparable<Point>
    {
        #region Properties

        /// <summary>
        /// Represents the X co-ordinate of point.
        /// </summary>
        public double X { get; protected set; }

        /// <summary>
        /// Represents the Y co-ordinate of point.
        /// </summary>
        public double Y { get; protected set; }

        #endregion

        #region Constructors

        /// <summary>
        /// Creates an instance of <see cref="Research.MapReduce.Samples.KMeansClustering.Point"/>.
        /// </summary>
        public Point() { }

        /// <summary>
        /// Creates an instance of <see cref="Research.MapReduce.Samples.KMeansClustering.Point"/>.
        /// </summary>
        /// <param name="value">The CSV string containing both the co-ordinates of point.</param>
        public Point(string value)
        {
            string[] splitVal = value.Split(',');
            X = double.Parse(splitVal[0]);
            Y = double.Parse(splitVal[1]);
        }

        /// <summary>
        /// Creates an instance of <see cref="Research.MapReduce.Samples.KMeansClustering.Point"/>.
        /// </summary>
        /// <param name="value1">Value of X co-ordinate of the point.</param>
        /// <param name="value2">Value of Y co-ordinate of the point.</param>
        public Point(string value1, string value2)
        {
            X = double.Parse(value1);
            Y = double.Parse(value2);
        }

        #endregion

        #region Methods

        /// <summary>
        /// Checks for equality of two instances of <see cref="Research.MapReduce.Samples.
        /// KMeansClustering.Point" />. Returns True if both are equal, returns Flase otherwise.
        /// </summary>
        /// <param name="other">The point with which this <see cref="Research.MapReduce.Samples.
        /// KMeansClustering.Point" /> object is to be compared.</param>
        /// <returns>A boolean value indicating whether the two <see cref="Research.MapReduce.Samples.
        /// KMeansClustering.Point" />s are equal or not.</returns>
        public bool Equals(Point other)
        {
            return this.X == other.X && this.Y == other.Y;
        }

        /// <summary>
        /// Calculates hash code of the <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />.
        /// </summary>
        /// <returns>The hash code of the <see cref="Research.MapReduce.Samples.KMeansClustering.Point" 
        /// />.</returns>
        public override int GetHashCode()
        {
            return Convert.ToInt32((this.X + this.Y) % (double)int.MaxValue);
        }

        /// <summary>
        /// Gets comma separated string representation of a <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />.
        /// </summary>
        /// <returns>The string representation of the <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />.
        /// </returns>
        public override string ToString()
        {
            return string.Format("{0},{1}", X, Y);
        }

        /// <summary>
        /// Compares two instances of <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />.
        /// </summary>
        /// <param name="other">The <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />
        /// with which the current <see cref="Research.MapReduce.Samples.KMeansClustering.Point" /> is to be 
        /// compared.</param>
        /// <returns>An integer indicating whether this <see cref="Research.MapReduce.Samples.KMeansClustering.Point" /> 
        /// is less than (-1), equal (0) to or greater (> 1) than the 'other' point.</returns>
        public int CompareTo(Point other)
        {
            int xComparison = this.X.CompareTo(other.X);
            if (xComparison != 0)
            {
                return xComparison;
            }
            return this.Y.CompareTo(other.Y);
        }

        /// <summary>
        /// Serializes the current <see cref="Research.MapReduce.Samples.KMeansClustering.Point" />.        
        /// </summary>
        /// <returns>A byte array representation of the current 
        /// <see cref="Research.MapReduce.Samples.KMeansClustering.Point"/>.</returns>
        public virtual byte[] ToBytes()
        {
            byte[] data = null;
            using (MemoryStream ms = new MemoryStream())
            {
                using (BinaryWriter bw = new BinaryWriter(ms))
                {
                    bw.Write(X);
                    bw.Write(Y);
                }
                data = ms.ToArray();
            }
            return data; ;
        }

        /// <summary>
        /// Updates the current <see cref="Research.MapReduce.Samples.KMeansClustering.Point"/> by
        /// getting the new values from the specified byte array containing the serialized data.
        /// </summary>        
        /// <param name="data">The byte array containing the serialized value.</param>
        public virtual void FromBytes(byte[] data)
        {
            using (MemoryStream ms = new MemoryStream(data))
            {
                using (BinaryReader br = new BinaryReader(ms))
                {
                    X = br.ReadDouble();
                    Y = br.ReadDouble();
                }
            }
        }

        #endregion
    }
}
