﻿/*
 * FILE:        Program.cs
 *                                                                      
 * DESCRIPTION: The file contains the implementation for a client application for running KMeansClustering sample.
 *
 * VERSION:     v1.0, updated July 2011.
 *
 * Copyright © 2011 Microsoft Corporation. All rights reserved.
 */

using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using Research.MapReduce.Core;
using Research.MapReduce.Samples.KMeansClustering.Properties;
using System.Diagnostics;

namespace Research.MapReduce.Samples.KMeansClustering
{
    class Program
    {
        #region Fields

        private const string PackageName = "KMeansClusteringSample";
        private static string masterConnectionString;
        private static CloudStorageAccount storageAccount;
        private static Dictionary<string, string> controllerArgs;

        #endregion

        #region Methods

        static void Main(string[] args)
        {
            // Read values of 'masterConnectionString' and 'storageAccount' from app.config.
            ValidateAndGetConfiguration();

            if (args.Length == 0)
            {
                Console.WriteLine();
                Console.WriteLine(@"Usage:- -InputDataLocation::<value> -OutputDataLocation::<value> -Mappers::<value> -Reducers::<value> -ClusterColumns::<value> [-Clusters::<value> | -InitialClustersLocation::<value>] -JobTimeoutInMinutes::<value>");
                args = GetDefaultInputArgs();
                Console.WriteLine();
                Console.WriteLine("Using default arguments >> {0}", args.GetString());
                Console.WriteLine();
            }

            // Populate controller parameter values from input arguments.
            controllerArgs = new Dictionary<string, string>();
            foreach (string arg in args)
            {
                string[] argArray = arg.Split(new string[] { "::" }, StringSplitOptions.
                    RemoveEmptyEntries);
                controllerArgs.Add(argArray[0].TrimStart('-'), argArray[1]);
            }

            // Submit application for execution and wait till it completes.
            // Note:- Assuming that it will take around 20 iterations to get the clusters, using a 
            // timeout that is slightly greater than total time for all KMeans jobs. Use a larger
            // timeout if other applications have already been submitted before this one.
            int timeoutInMinutes = int.Parse(controllerArgs[ParameterNames.JobTimeoutInMinutes]) * 20 + 2;
            SubmitAndWaitForApplication(timeoutInMinutes);

            Console.WriteLine("Press <ENTER> to exit.");
            Console.ReadLine();
        }

        /// <summary>
        /// Reads and validates values of 'masterConnectionString' and 'StorageConnectionString' 
        /// from app.config.         
        /// </summary>
        private static void ValidateAndGetConfiguration()
        {
            masterConnectionString = ConfigurationManager.AppSettings["MasterConnectionString"];
            string storageConnectionString = ConfigurationManager.AppSettings["MasterConnectionString"];

            if (string.IsNullOrEmpty(masterConnectionString) || string.IsNullOrWhiteSpace(masterConnectionString))
            {
                throw new InvalidOperationException("MasterConnectionString configuration missing.");
            }
            if (string.IsNullOrEmpty(storageConnectionString) || string.IsNullOrWhiteSpace(storageConnectionString))
            {
                throw new InvalidOperationException("StorageConnectionString configuration missing.");
            }

            if (!CloudStorageAccount.TryParse(masterConnectionString, out storageAccount))
            {
                throw new InvalidOperationException("MasterConnectionString incorrect.");
            }

            if (!CloudStorageAccount.TryParse(storageConnectionString, out storageAccount))
            {
                throw new InvalidOperationException("StorageConnectionString incorrect.");
            }
        }

        /// <summary>
        /// Gets default input arguments for running the sample.
        /// </summary>
        /// <returns>String array containing the arguments for running the sample.</returns>
        private static string[] GetDefaultInputArgs()
        {
            string inputContainerName = "clustering-input-" + Environment.UserName;
            string outputContainerName = "clustering-output-" + Environment.UserName;

            string[] args = new string[7];

            // Arg0 - InputDataLocation
            CloudBlobContainer container = GetInitializedContainer(inputContainerName);
            CloudBlob blob = container.GetBlobReference("sampleinput.csv");
            blob.UploadText(Resources.sampleinput);

            // Arg0 - InputDataLocation
            args[0] = string.Format("-{0}::{1}", ParameterNames.InputDataLocation, blob.Uri);

            // Arg1 - OutputDataLocation
            container = GetInitializedContainer(outputContainerName);
            args[1] = string.Format("-{0}::{1}", ParameterNames.OutputDataLocation, container.Name);

            // Arg2 - Mappers
            args[2] = string.Format("-{0}::{1}", ParameterNames.Mappers, 4);

            // Arg3 - Reducers
            args[3] = string.Format("-{0}::{1}", ParameterNames.Reducers, 2);

            // Arg4 - Clusters
            args[4] = string.Format("-{0}::{1}", ParameterNames.Clusters, 4);

            // Arg5 - ClusterColumns
            args[5] = string.Format("-{0}::{1}", ParameterNames.ClusterColumns, "Duration,Time");

            // Arg6 - JobTimeoutInMinutes
            args[6] = string.Format("-{0}::{1}", ParameterNames.JobTimeoutInMinutes, 2);

            return args;
        }

        /// <summary>
        /// Creates a CloudBlobContainer if it does not already exist. Cleans the container if it 
        /// exists.
        /// </summary>
        /// <param name="name">Name of the container to be initialized.</param>
        /// <returns>CloudBlobContainer object.</returns>
        private static CloudBlobContainer GetInitializedContainer(string name)
        {
            CloudBlobContainer container = storageAccount.CreateCloudBlobClient().GetContainerReference(name);
            container.CreateIfNotExist();
            container.Clean();
            return container;
        }

        /// <summary>
        /// Submits application for execution and waits till it completes.
        /// </summary>
        /// <param name="timeoutInMinutes">Timeout value for the application in minutes</param>
        private static void SubmitAndWaitForApplication(int timeoutInMinutes)
        {
            MapReduceClient client = new MapReduceClient(PackageName, Guid.NewGuid().
                ToString("N"), typeof(KMeansClusteringController).AssemblyQualifiedName,
                controllerArgs);
            Stopwatch sw = new Stopwatch();
            sw.Start();
            client.Submit(masterConnectionString);

            Console.WriteLine("Application submitted successfully...");

            Console.WriteLine("Waiting for completion...");

            bool concluded = client.WaitForCompletion(masterConnectionString, TimeSpan.
                FromMinutes(timeoutInMinutes), TimeSpan.FromSeconds(5));
            sw.Stop();
            if (concluded)
            {
                if (client.Succeeded)
                {
                    Console.WriteLine("Application completed successfully. Execution Time: "+((double)sw.ElapsedMilliseconds)/1000+" seconds.");                    
                    Console.WriteLine();

                    string numberOfIterations = client.Results.Where(tuple => tuple.Key ==
                        ResultNames.NumberOfIterations).First().Value;
                    Console.WriteLine("Number of iterations taken to compute clusters:: [{0}]",
                        numberOfIterations);
                    Console.WriteLine();

                    Console.WriteLine("Clusters are::");
                    Console.WriteLine();
                    Console.WriteLine("ClusterID, Cluster Columns [{0}]", controllerArgs[
                        ParameterNames.ClusterColumns]);
                    string finalClustersBlobUri = client.Results.Where(tuple => tuple.Key ==
                        ResultNames.FinalClustersBlobUri).First().Value;
                    CloudBlob blob = storageAccount.CreateCloudBlobClient().GetBlobReference(finalClustersBlobUri);
                    using (StreamReader sr = new StreamReader(blob.OpenRead()))
                    {
                        Console.WriteLine(sr.ReadToEnd());
                    }
                }
                else
                {
                    Console.WriteLine("Application failed due to: \n {0}", client.FailReason);
                }
            }
            else
            {
                Console.WriteLine("Application wait timeout elapsed.");
            }
        }

        #endregion
    }
}
