﻿/*
 * FILE:        WeightedPoint.cs
 *                                                                      
 * DESCRIPTION: The file contains implementation for representing the combination/sum of one or more instances of 'Point'.
 *
 * VERSION:     v1.0, updated July 2011.
 *
 * Copyright © 2011 Microsoft Corporation. All rights reserved.
 */

using System.IO;

namespace Research.MapReduce.Samples.KMeansClustering
{
    /// <summary>
    /// Represents the combination/sum of one or more instances of <see cref="Research.MapReduce.Samples.KMeansClustering.Point"/>.
    /// </summary>
    public class WeightedPoint : Point
    {
        #region Properties

        /// <summary>
        /// Represents the weight of the associated point co-ordinates.
        /// </summary>
        public int Weight { get; private set; }

        #endregion

        #region Constructors

        /// <summary>
        /// Creates an instance of <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint"/>.
        /// </summary>
        public WeightedPoint() { }

        /// <summary>
        /// Creates an instance of <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint"/>.
        /// </summary>
        /// <param name="value">The CSV string containing both the co-ordinates of a point.</param>
        public WeightedPoint(string value)
            : base(value)
        {
            Weight = 1;
        }

        /// <summary>
        /// Creates an instance of <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint"/>.
        /// </summary>
        /// <param name="value1">Value of X co-ordinate of the point.</param>
        /// <param name="value2">Value of Y co-ordinate of the point.</param>
        /// <param name="weight">Value of weight associated with the points.</param>
        public WeightedPoint(double value1, double value2, int weight)
        {
            X = value1;
            Y = value2;
            Weight = weight;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Adds an instance of <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint" />
        /// to another.
        /// </summary>
        /// <param name="val">The <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint" />
        /// to be added.</param>
        public void Add(WeightedPoint val)
        {
            X += val.X;
            Y += val.Y;
            Weight += val.Weight;
        }

        /// <summary>
        /// Calculates and returns the average of a <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint" />.
        /// </summary>
        /// <returns>The averaged <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint" />.
        /// </returns>
        public WeightedPoint Average()
        {
            WeightedPoint p = new WeightedPoint();
            p.X = X / (double)Weight;
            p.Y = Y / (double)Weight;
            p.Weight = 1;
            return p;
        }

        /// <summary>
        /// Updates the current <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint"/>
        /// by getting the new values from the specified byte array containing the serialized data.
        /// </summary>        
        /// <param name="data">The byte array containing the serialized value.</param>
        public override void FromBytes(byte[] data)
        {
            using (MemoryStream ms = new MemoryStream(data))
            {
                using (BinaryReader br = new BinaryReader(ms))
                {
                    X = br.ReadDouble();
                    Y = br.ReadDouble();
                    Weight = br.ReadInt32();
                }
            }
        }

        /// <summary>
        /// Serializes the current <see cref="Research.MapReduce.Samples.KMeansClustering.WeightedPoint" />.
        /// </summary>
        /// <returns>A byte array representation of the current 
        /// <see cref="Research.MapReduce.Samples.KMeansClustering.Point"/>.</returns>
        public override byte[] ToBytes()
        {
            byte[] data = null;
            using (MemoryStream ms = new MemoryStream())
            {
                using (BinaryWriter bw = new BinaryWriter(ms))
                {
                    bw.Write(X);
                    bw.Write(Y);
                    bw.Write(Weight);
                }
                data = ms.ToArray();
            }
            return data;
        }

        #endregion
    }
}
