﻿//------------------------------------------------------------------------------------------------- 
// <copyright file="BlobResultCollection.cs" company="Microsoft">
// Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-------------------------------------------------------------------------------------------------

namespace Research.MapReduce.Samples.OutlierDetection
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Diagnostics.CodeAnalysis;
    using Microsoft.WindowsAzure.StorageClient;
    using Research.MapReduce.Core;

    /// <summary>
    /// This class contains logic of operation performed on blob result.
    /// </summary>
    /// <typeparam name="K">The key for blob result.</typeparam>
    /// <typeparam name="V">The value of blob result.</typeparam>
    [Serializable]
    public sealed class BlobResultCollection<K, V> : IReduceResult<K, V>
    {
        /// <summary>
        /// Delimiter specified by the user.
        /// </summary>
        private string keyValueDelimiter = " ";

        /// <summary>
        /// Gets or sets the CloudClient.
        /// </summary>
        public CloudClient CloudClient { get; set; }

        /// <summary>
        /// Gets or sets the result blob Uri.
        /// </summary>
        [SuppressMessage("Microsoft.Design", "CA1056:UriPropertiesShouldNotBeStrings", Justification = "BlobUri should be a string.")]
        public string BlobUri { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether to write the key to the blob.
        /// </summary>
        public bool IgnoreKey { get; set; }

        /// <summary>
        /// Gets the buffer
        /// </summary>
        [SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays", Justification = "Property needs to return array.")]
        public byte[] Buffer { get; private set; }

        /// <summary>
        /// Gets or sets the delimiter to be used by the separate key and value.
        /// This is to help users to store Csv as values.
        /// </summary>
        public string KeyValueDelimiter
        {
            get { return this.keyValueDelimiter; }
            set { this.keyValueDelimiter = value; }
        }

        /// <summary>
        /// Gets the result reader from for a blob writer.
        /// </summary>
        /// <returns>Returns the reader.</returns>
        public IRecordReader<K, V> GetResultReader
        {
            get
            {
                CloudBlob blob = CloudClient.BlobClient.GetBlobReference(this.BlobUri);
                return new BlobResultReader<K, V>(blob, this.keyValueDelimiter, this.IgnoreKey);
            }
        }

        /// <summary>
        /// Get the result enumerator from for a blob writer.
        /// </summary>
        /// <returns>returns the enumerator.</returns>
        public IEnumerator<KeyValuePair<K, V>> GetEnumerator()
        {
            return new BlobResultEnumerator<K, V>(this);
        }

        /// <summary>
        /// Get the result enumerator from for a blob writer.
        /// </summary>
        /// <returns>returns the enumerator.</returns>
        IEnumerator IEnumerable.GetEnumerator()
        {
            return new BlobResultEnumerator<K, V>(this);
        }
    }
}