﻿//------------------------------------------------------------------------------------------------- 
// <copyright file="BlobResultEnumerator.cs" company="Microsoft">
// Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-------------------------------------------------------------------------------------------------

namespace Research.MapReduce.Samples.OutlierDetection
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.IO;
    using Microsoft.WindowsAzure.StorageClient;

    /// <summary>
    /// This class contains logic of operation performed on blob result.
    /// </summary>
    /// <typeparam name="K">The key for blob result.</typeparam>
    /// <typeparam name="V">The value of blob result.</typeparam>
    public sealed class BlobResultEnumerator<K, V> : IEnumerator<KeyValuePair<K, V>>
    {
        /// <summary>
        /// The reader for reading blob result.
        /// </summary>
        private StreamReader reader;

        /// <summary>
        /// The key value pair.
        /// </summary>
        private KeyValuePair<K, V> current;

        /// <summary>
        /// Custom delimiter for the key value pair.
        /// </summary>
        private string keyValueDelimiter;

        /// <summary>
        /// Flag that indicates weather to write the key to the blob.
        /// </summary>
        private bool ignoreKey;

        /// <summary>
        /// Initializes a new instance of the BlobResultEnumerator class.
        /// </summary>
        /// <param name="result">Returns the blob result.</param>
        public BlobResultEnumerator(BlobResultCollection<K, V> result)
        {
            CloudBlob blob = result.CloudClient.BlobClient.GetBlobReference(result.BlobUri);
            if (result.Buffer != null)
            {
                this.reader = new StreamReader(new MemoryStream(result.Buffer));
            }
            else
            {
                this.reader = new StreamReader(blob.OpenRead());
            }

            this.ignoreKey = result.IgnoreKey;
            this.keyValueDelimiter = result.KeyValueDelimiter;
        }

        /// <summary>
        /// Gets the current key value pair.
        /// </summary>
        public KeyValuePair<K, V> Current
        {
            get { return this.current; }
        }

        /// <summary>
        /// Gets the current enumerator.
        /// </summary>
        object IEnumerator.Current
        {
            get { return this.current; }
        }

        /// <summary>
        /// The method to dispose reader.
        /// </summary>
        public void Dispose()
        {
            if (this.reader != null)
            {
                this.reader.Dispose();
            }
        }

        /// <summary>
        /// gets the next record.
        /// </summary>
        /// <returns>Returns the bool value.</returns>
        public bool MoveNext()
        {
            if (this.reader.EndOfStream)
            {
                return false;
            }

            string record = this.reader.ReadLine();

            if (!this.ignoreKey)
            {
                string key = record.Substring(0, record.IndexOf(','));
                string value = record.Substring(record.IndexOf(',') + 1);
                this.current = new KeyValuePair<K, V>((K)TypeDescriptor.GetConverter(typeof(K)).ConvertFromString(key), (V)TypeDescriptor.GetConverter(typeof(V)).ConvertFromString(value));
            }
            else
            {
                this.current = new KeyValuePair<K, V>((K)default(K), (V)TypeDescriptor.GetConverter(typeof(V)).ConvertFromString(record));
            }

            return true;
        }

        /// <summary>
        /// The methods to reset.
        /// </summary>
        public void Reset()
        {
            throw new NotImplementedException();
        }
    }
}