﻿//------------------------------------------------------------------------------------------------- 
// <copyright file="BlobResultReader.cs" company="Microsoft">
// Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-------------------------------------------------------------------------------------------------

namespace Research.MapReduce.Samples.OutlierDetection
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.IO;
    using Microsoft.WindowsAzure.StorageClient;
    using Research.MapReduce.Core;

    /// <summary>
    /// This class contains logic of reading records on blob result.
    /// </summary>
    /// <typeparam name="K">The key value.</typeparam>
    /// <typeparam name="V">The value.</typeparam>
    public sealed class BlobResultReader<K, V> : IRecordReader<K, V>
    {
        /// <summary>
        /// Cloud blob where the results are stored.
        /// </summary>
        private CloudBlob blob;

        /// <summary>
        /// Custom delimiter for the key value pair.
        /// </summary>
        private string keyValueDelimiter = " ";

        /// <summary>
        /// Initializes a new instance of the BlobResultReader class.
        /// </summary>
        /// <param name="blob">The blob address.</param>
        public BlobResultReader(CloudBlob blob)
        {
            this.blob = blob;
        }

        /// <summary>
        /// Initializes a new instance of the BlobResultReader class.
        /// </summary>
        /// <param name="blob">Cloud blob reference.</param>
        /// <param name="delimiter">Custom delimiter.</param>
        /// <param name="ignoreKey">Flag that indicates weather to ignore the key.</param>
        public BlobResultReader(CloudBlob blob, string delimiter, bool ignoreKey)
        {
            this.blob = blob;
            this.keyValueDelimiter = delimiter;
            this.IgnoreKey = ignoreKey;
        }

        /// <summary>
        /// Gets or sets a value indicating whether to write the key to the blob or not.
        /// </summary>
        public bool IgnoreKey { get; set; }

        /// <summary>
        /// Gets or sets the delimiter to be used by the separate key and value.
        /// This is to help users to store Csv as values.
        /// </summary>
        public string KeyValueDelimiter
        {
            get { return this.keyValueDelimiter; }
            set { this.keyValueDelimiter = value; }
        }

        /// <summary>
        /// Get the records from the result blob.
        /// </summary>
        /// <returns>Collection of key value pair.</returns>
        public IEnumerable<KeyValuePair<K, V>> GetRecords()
        {
            using (StreamReader sr = new StreamReader(this.blob.OpenRead()))
            {
                while (!sr.EndOfStream)
                {
                    string[] record = sr.ReadLine().Split(
                        new string[] { this.keyValueDelimiter },
                        StringSplitOptions.RemoveEmptyEntries);

                    if (this.IgnoreKey)
                    {
                        // If ignore key then remove the key part
                        // But we need a default value here.
                        yield return new KeyValuePair<K, V>(
                            GetDefaultValue(),
                            (V)TypeDescriptor.GetConverter(typeof(V)).ConvertFromString(record[0]));
                    }
                    else
                    {
                        yield return new KeyValuePair<K, V>(
                            (K)TypeDescriptor.GetConverter(typeof(K)).ConvertFromString(record[0]),
                          (V)TypeDescriptor.GetConverter(typeof(V)).ConvertFromString(record[1]));
                    }
                }
            }
        }

        /// <summary>
        /// Get the default value.
        /// </summary>
        /// <returns>Default value of the type specified as key.</returns>
        private static K GetDefaultValue()
        {
            if (typeof(K).IsValueType)
            {
                return default(K);
            }
            else
            {
                return Activator.CreateInstance<K>();
            }
        }
    }
}
