﻿//------------------------------------------------------------------------------------------------- 
// <copyright file="FindAttributesCountMapper.cs" company="Microsoft">
// Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-------------------------------------------------------------------------------------------------

namespace Research.MapReduce.Samples.OutlierDetection
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Research.MapReduce.Core;
    using Research.MapReduce.Samples.OutlierDetection.Properties;

    /// <summary>
    /// Split the attributes.
    /// </summary>
    public sealed class FindAttributesCountMapper : IMapper<int, string, string, int>
    {
        /// <summary>
        /// Used to store the indexes of the data columns in the input file.
        /// </summary>
        private int[] columnIndexes;

        /// <summary>
        /// Used to store the number of user specified columns.
        /// </summary>
        private int numberOfUserSpecifiedColumns;

        /// <summary>
        /// Configure Mapper.
        /// </summary>
        /// <param name="context">MapContext to get the user inputs.</param>
        public void Configure(MapContext<int, string> context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context", Resources.MapContextCannotBeNull);
            }

            if (string.IsNullOrWhiteSpace(context.JobParameters[ParameterNames.CsvHeader]))
            {
                throw new ArgumentNullException(Resources.CsvHeaderCannotBeEmpty);
            }

            if (string.IsNullOrWhiteSpace(context.JobParameters[ParameterNames.ColumnNames]))
            {
                throw new ArgumentNullException(Resources.ColumnNamesCannotBeEmpty);
            }

            var columnNames = context.JobParameters[ParameterNames.ColumnNames].Split(',').Select(x => x.Trim().ToUpperInvariant()).ToArray();
            this.numberOfUserSpecifiedColumns = columnNames.Length;

            // Get all the column headers.
            var metadata = context.JobParameters[ParameterNames.CsvHeader];
            var columnHeaders = metadata.Split(',').Select(x => x.Trim().ToUpperInvariant()).ToArray();
            this.columnIndexes = new int[this.numberOfUserSpecifiedColumns];
            int columnIndex;
            var columnsNotFound = new StringBuilder();

            for (int index = 0; index < this.numberOfUserSpecifiedColumns; index++)
            {
                columnIndex = Array.IndexOf(columnHeaders, columnNames[index]);
                if (columnIndex >= 0)
                {
                    this.columnIndexes[index] = columnIndex;
                }
                else
                {
                    columnsNotFound.Append(columnNames[index]);
                    columnsNotFound.Append(", ");
                }
            }

            if (columnsNotFound.Length > 0)
            {
                columnsNotFound.Remove(columnsNotFound.Length - 2, 2);
                throw new ArgumentException(columnsNotFound + " could not be found.");
            }
        }

        /// <summary>
        /// Map method sends the attribute value in each user selected column with count as 1.
        /// </summary>
        /// <param name="key">Integer key.</param>
        /// <param name="value">Comma separated column values.</param>
        /// <param name="context">MapContext to get the user inputs.</param>
        /// <returns>Filtered datarows.</returns>
        public IEnumerable<KeyValuePair<string, int>> Map(int key, string value, MapContext<int, string> context)
        {
            var stringvalues = value.Split(',');
            int columnIndex;
            string stringvalue;

            for (int index = 0; index < this.numberOfUserSpecifiedColumns; index++)
            {
                columnIndex = this.columnIndexes[index];
                stringvalue = stringvalues[columnIndex];
                yield return new KeyValuePair<string, int>(index.ToString() + "_" + stringvalue.ToUpperInvariant(), 1);
            }
        }
    }
}