﻿//------------------------------------------------------------------------------------------------- 
// <copyright file="FindOutliersMapper.cs" company="Microsoft">
// Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-------------------------------------------------------------------------------------------------

namespace Research.MapReduce.Samples.OutlierDetection
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.IO;
    using System.Linq;
    using System.Text;
    using Microsoft.WindowsAzure.StorageClient;
    using Research.MapReduce.Core;
    using Research.MapReduce.Samples.OutlierDetection.Properties;

    /// <summary>
    /// Calculate AVF value.
    /// </summary>
    public class FindOutliersMapper : IMapper<int, string, int, AttributeValueFrequencyRecord>
    {
        /// <summary>
        /// Used to store the indexes of the data columns in the input file.
        /// </summary>
        private int[] columnIndexes;

        /// <summary>
        /// Used to store the number of user specified columns.
        /// </summary>
        private int numberOfUserSpecifiedColumns;

        /// <summary>
        /// Used to store the count of each attribute value.
        /// </summary>
        private Dictionary<string, int> frequencies = new Dictionary<string, int>();

        /// <summary>
        /// Configure Mapper.
        /// </summary>
        /// <param name="context">MapContext to get the user inputs.</param>
        public void Configure(MapContext<int, string> context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context", Resources.MapContextCannotBeNull);
            }

            if (string.IsNullOrWhiteSpace(context.JobParameters[ParameterNames.CsvHeader]))
            {
                throw new ArgumentNullException(Resources.CsvHeaderCannotBeEmpty);
            }

            if (string.IsNullOrWhiteSpace(context.JobParameters[ParameterNames.ColumnNames]))
            {
                throw new ArgumentNullException(Resources.ColumnNamesCannotBeEmpty);
            }

            if (string.IsNullOrWhiteSpace(context.JobParameters[ParameterNames.StorageConnectionString]))
            {
                throw new ArgumentNullException(Resources.StorageConnectionStringCannotBeEmpty);
            }

            if (string.IsNullOrWhiteSpace(context.JobParameters[ParameterNames.FindAttributesCountOutputContainerName]))
            {
                throw new ArgumentNullException(Resources.FindAttributesCountOutputContainerNameCannotBeEmpty);
            }

            // Get all the column headers.
            var metadata = context.JobParameters[ParameterNames.CsvHeader];
            var columnHeaders = metadata.Split(',').Select(x => x.Trim().ToUpperInvariant()).ToArray();

            var columnNames = context.JobParameters[ParameterNames.ColumnNames].Split(',').Select(x => x.Trim().ToUpperInvariant()).ToArray();
            this.numberOfUserSpecifiedColumns = columnNames.Length;
            this.columnIndexes = new int[this.numberOfUserSpecifiedColumns];
            int columnIndex;
            var columnsNotFound = new StringBuilder();

            for (int index = 0; index < this.numberOfUserSpecifiedColumns; index++)
            {
                columnIndex = Array.IndexOf(columnHeaders, columnNames[index]);
                if (columnIndex >= 0)
                {
                    this.columnIndexes[index] = columnIndex;
                }
                else
                {
                    columnsNotFound.Append(columnNames[index]);
                    columnsNotFound.Append(", ");
                }
            }

            if (columnsNotFound.Length > 0)
            {
                columnsNotFound.Remove(columnsNotFound.Length - 2, 2);
                throw new ArgumentException(columnsNotFound + " could not be found.");
            }

            var storageConnectionString = context.JobParameters[ParameterNames.StorageConnectionString];
            var findAttributesCountOutputContainerName = context.JobParameters[ParameterNames.FindAttributesCountOutputContainerName];

            CloudClient cloudClient = null;

            if (!string.IsNullOrWhiteSpace(storageConnectionString))
            {
                cloudClient = new CloudClient(storageConnectionString);
            }

            var findAttributesCountOutputContainer = cloudClient.BlobClient.GetContainerReference(findAttributesCountOutputContainerName);
            try
            {
                findAttributesCountOutputContainer.FetchAttributes();
            }
            catch (StorageClientException e)
            {
                if (e.ErrorCode == StorageErrorCode.ResourceNotFound)
                {
                    throw new ArgumentException(Resources.OutputBlobContainerInvalidErrorMessage);
                }
                else
                {
                    throw;
                }
            }

            this.DownloadAttributesCount(findAttributesCountOutputContainer);
        }

        /// <summary>
        /// Map method to filter datarows based on the value of a column using regular expression.
        /// </summary>
        /// <param name="key">Integer key.</param>
        /// <param name="value">Comma separated column values.</param>
        /// <param name="context">MapContext to get the user inputs.</param>
        /// <returns>Filtered datarows.</returns>
        public IEnumerable<KeyValuePair<int, AttributeValueFrequencyRecord>> Map(int key, string value, MapContext<int, string> context)
        {
            var stringvalues = value.Split(',');
            int columnIndex;
            string stringvalue;
            int avf = 0;

            for (int index = 0; index < this.numberOfUserSpecifiedColumns; index++)
            {
                columnIndex = this.columnIndexes[index];
                stringvalue = stringvalues[columnIndex];
                avf += this.frequencies[index.ToString() + "_" + stringvalue.ToUpperInvariant()];
            }

            yield return new KeyValuePair<int, AttributeValueFrequencyRecord>(1, new AttributeValueFrequencyRecord(value, avf));
        }

        /// <summary>
        /// Used to attributes with their frequency of occurence.
        /// </summary>
        /// <param name="line">ColumnIndex (separated by _) AttributeValue (separated by ,) Frequency.</param>
        /// <param name="property">ColumnIndex (separated by _) AttributeValue</param>
        /// <param name="frequency">Frequency of occurence.</param>
        private static void GetFrequencies(string line, out string property, out int frequency)
        {
            string[] parts = line.Split(',');
            property = parts[0];
            frequency = Int32.Parse(parts[1], CultureInfo.InvariantCulture);
        }

        /// <summary>
        /// Download centroids from blob container and add them to canopyCentres variable.
        /// </summary>
        /// <param name="findAttributesCountOutputContainer">Output blob container of phase 1.</param>
        private void DownloadAttributesCount(CloudBlobContainer findAttributesCountOutputContainer)
        {
            var findAttributesCountBlobs = findAttributesCountOutputContainer.ListBlobs();
            var findAttributesCountBlobsCount = findAttributesCountBlobs.Count();
            string property;
            int frequency;

            for (int blobIndex = 0; blobIndex < findAttributesCountBlobsCount; blobIndex++)
            {
                var blob = (CloudBlockBlob)findAttributesCountBlobs.ElementAt(blobIndex);
                using (TextReader tr = new StreamReader(blob.OpenRead()))
                {
                    string line;
                    while ((line = tr.ReadLine()) != null)
                    {
                        GetFrequencies(line, out property, out frequency);
                        if (this.frequencies.ContainsKey(property))
                        {
                            this.frequencies[property] += frequency;
                        }
                        else
                        {
                            this.frequencies[property] = frequency;
                        }
                    }
                }
            }
        }
    }
}