﻿//------------------------------------------------------------------------------------------------- 
// <copyright file="OutlierDetectionController.cs" company="Microsoft">
// Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-------------------------------------------------------------------------------------------------

namespace Research.MapReduce.Samples.OutlierDetection
{
    using System;
    using System.Globalization;
    using System.IO;
    using System.Linq;
    using System.Threading.Tasks;
    using Microsoft.WindowsAzure.StorageClient;
    using Research.MapReduce.Core;
    using Research.MapReduce.Library;
    using Research.MapReduce.Samples.OutlierDetection.Properties;

    /// <summary>
    /// Outlier detection startegy based on the Attribute value Frequency (AVF) approach, a high speed, scalable outlier detection method for categorical data that is inherently easy to parallelize.
    /// </summary>
    [Controller(Name = "Outlier Detection - Attribute value Frequency", Description = "Outlier detection startegy based on the Attribute value Frequency (AVF) approach, a high speed, scalable outlier detection method for categorical data that is inherently easy to parallelize.")]
    public sealed class OutlierDetectionController : Research.MapReduce.Core.Controller
    {
        /// <summary>
        /// Gets or sets names of the columns in the input file.
        /// </summary>
        [ControllerParameter(Name = ParameterNames.ColumnNames, Description = "Name of the column in the input file.", IsRequired = true, DefaultValue = "")]
        public string ColumnNames { get; set; }

        /// <summary>
        /// Gets or sets the number of outliers need to be displayed in the output.
        /// </summary>
        [ControllerParameter(Name = ParameterNames.NumberOfOutliers, Description = "Number of outliers need to be displayed in the output.", IsRequired = true, DefaultValue = "")]
        public int NumberOfOutliers { get; set; }

        /// <summary>
        /// Gets or sets the duration within which a phase of outlier detection job must complete.
        /// </summary>
        [ControllerParameter(Name = ParameterNames.JobTimeoutInMinutes, Description = "Duration within which a phase of outlier detection job must complete.", DefaultValue = "5", IsRequired = true)]
        public int JobTimeoutInMinutes { get; set; }

        /// <summary>
        /// Overrides Run method of the Controller base class
        /// </summary>
        public override void Run()
        {
            this.ValidateInputParameters();

            var findAttributeCountOutputContainerName = "find-attribute-count-" + Guid.NewGuid().ToString("N");
            var findAttributeCountJob = this.GetJobForFindAttributeCount(findAttributeCountOutputContainerName);
            var task = Task.Factory.StartNew(() => findAttributeCountJob.Run());

            task.Wait();

            var findOutliersOutputContainerName = "find-outliers-" + Guid.NewGuid().ToString("N");
            var findOutliersJob = this.GetJobForFindOutliers(findAttributeCountOutputContainerName);
            task = Task.Factory.StartNew(() => findOutliersJob.Run());

            task.Wait();

            var outliers = findOutliersJob.GetReduceOutputs<int, string>();
            var finaloutliers = outliers.SelectMany(kvp => kvp).OrderBy(kvp => Convert.ToInt16(kvp.Value.Substring(kvp.Value.LastIndexOf(',') + 1))).Take(this.NumberOfOutliers);
            CloudBlobContainer outputBlobContainer = this.CloudClient.BlobClient.GetContainerReference(this.OutputDataLocation);
            var outputBlob = outputBlobContainer.GetBlobReference("0");

            using (StreamWriter writer = new StreamWriter(outputBlob.OpenWrite()))
            {
                var dataPartitioner = new BlobCsvPartitioner(CloudClient, this.InputDataLocation, RequestedNumberOfMappers);
                writer.WriteLine(dataPartitioner.Header + ",AVF Score");

                foreach (var keyValuePair in finaloutliers)
                {
                    writer.WriteLine(keyValuePair.Value);
                }
            }

            this.Results.Add(ResultNames.OutputContainer, this.OutputDataLocation);
        }

        /// <summary>
        /// Validates the input parameters.
        /// </summary>
        private void ValidateInputParameters()
        {
            if (string.IsNullOrEmpty(this.StorageConnectionString) || string.IsNullOrWhiteSpace(this.StorageConnectionString))
            {
                throw new ArgumentNullException(ParameterNames.StorageConnectionString);
            }

            if (string.IsNullOrEmpty(this.InputDataLocation) || string.IsNullOrWhiteSpace(this.InputDataLocation))
            {
                throw new ArgumentNullException("InputDataLocation");
            }
            else
            {
                this.ValidateInputDataLocation();
            }

            if (string.IsNullOrEmpty(this.OutputDataLocation) || string.IsNullOrWhiteSpace(
                this.OutputDataLocation))
            {
                throw new ArgumentNullException("OutputDataLocation");
            }
            else
            {
                this.ValidateOutputDataLocation();
            }

            if (RequestedNumberOfMappers <= 0)
            {
                throw new ArgumentException(Resources.NumberOfMappersInvalidErrorMessage);
            }

            if (RequestedNumberOfReducers <= 0)
            {
                throw new ArgumentException(Resources.NumberOfReducersInvalidErrorMessage);
            }

            if (string.IsNullOrWhiteSpace(this.ColumnNames))
            {
                throw new ArgumentNullException(Resources.ColumnNamesCannotBeEmpty);
            }

            if (this.NumberOfOutliers <= 0)
            {
                throw new ArgumentException(Resources.NumberOfOutliersShouldBeGreaterThanZero);
            }
        }

        /// <summary>
        /// Gets JobConfiguration required for find attribute count.
        /// </summary>
        /// <param name="outputBlobContainerName">Output blob container name.</param>
        /// <returns>Job required for Finding centroids.</returns>
        private Job GetJobForFindAttributeCount(string outputBlobContainerName)
        {
            var jobConfiguration = new JobConfiguration();
            var dataPartitioner = new BlobCsvPartitioner(CloudClient, this.InputDataLocation, RequestedNumberOfMappers);

            jobConfiguration.MapperType = typeof(FindAttributesCountMapper);
            jobConfiguration.MapOutputStorage = MapOutputStoreType.Local;
            jobConfiguration.KeyPartitioner = typeof(HashModuloKeyPartitioner<string, int>);
            jobConfiguration.ReducerType = typeof(FindAttributesCountReducer);

            jobConfiguration.ExceptionPolicy = new TerminateOnFirstException();
            jobConfiguration.JobTimeout = TimeSpan.FromMinutes(10);

            jobConfiguration.JobParameters.Add(ParameterNames.CsvHeader, dataPartitioner.Header);
            jobConfiguration.JobParameters.Add(ParameterNames.ColumnNames, this.ColumnNames);

            var job = new Job(jobConfiguration, this);
            job.DataPartitioner = dataPartitioner;
            job.RecordWriter = new OutlierDetectionWriter<string, int>
            {
                CloudClient = this.CloudClient,
                ContainerName = outputBlobContainerName,
                KeyValueDelimiter = ","
            };
            job.NoOfReduceTasks = this.RequestedNumberOfReducers;
            
            return job;
        }

        /// <summary>
        /// Gets JobConfiguration required for finding canopies for datapoints.
        /// </summary>
        /// <param name="findAttributeCountOutputContainerName">Outpoint container of the FindCentroids call.</param>
        /// <returns>Job required for Finding centroids.</returns>
        private Job GetJobForFindOutliers(string findAttributeCountOutputContainerName)
        {
            var jobConfiguration = new JobConfiguration();
            var dataPartitioner = new BlobCsvPartitioner(CloudClient, this.InputDataLocation, RequestedNumberOfMappers);

            jobConfiguration.MapperType = typeof(FindOutliersMapper);
            jobConfiguration.MapOutputStorage = MapOutputStoreType.Local;
            jobConfiguration.KeyPartitioner = typeof(HashModuloKeyPartitioner<int, AttributeValueFrequencyRecord>);
            jobConfiguration.ReducerType = typeof(FindOutliersReducer);

            jobConfiguration.ExceptionPolicy = new TerminateOnFirstException();
            jobConfiguration.JobTimeout = TimeSpan.FromMinutes(10);

            jobConfiguration.JobParameters.Add(ParameterNames.CsvHeader, dataPartitioner.Header);
            jobConfiguration.JobParameters.Add(ParameterNames.ColumnNames, this.ColumnNames);
            jobConfiguration.JobParameters.Add(ParameterNames.NumberOfOutliers, this.NumberOfOutliers.ToString(CultureInfo.InvariantCulture));
            jobConfiguration.JobParameters.Add(ParameterNames.StorageConnectionString, this.StorageConnectionString);
            jobConfiguration.JobParameters.Add(ParameterNames.FindAttributesCountOutputContainerName, findAttributeCountOutputContainerName);

            var job = new Job(jobConfiguration, this);
            job.DataPartitioner = dataPartitioner;
            job.RecordWriter = new OutlierDetectionWriter<int, string>
            {
                CloudClient = this.CloudClient,
                ContainerName = this.OutputDataLocation,
                IgnoreKey = true
            };
            job.NoOfReduceTasks = this.RequestedNumberOfReducers;
            job.GetReduceOutputsToController = true;           

            return job;
        }

        /// <summary>
        /// Validates the input parameter 'InputDataLocation'.
        /// </summary>
        private void ValidateInputDataLocation()
        {
            this.InputDataLocation.ValidateUriFormat("InputDataLocation");
            try
            {
                CloudBlob blob = this.CloudClient.BlobClient.GetBlobReference(this.
                    InputDataLocation);
                blob.FetchAttributes();
                if (blob.Properties.Length <= 0)
                {
                    throw new ArgumentException(
                    string.Format(CultureInfo.InvariantCulture, "Input location [{0}] is empty.", this.InputDataLocation));
                }
            }
            catch (StorageException ex)
            {
                throw new ArgumentException(
                    string.Format(CultureInfo.InvariantCulture, "Error occurred while trying to access input location [{0}]", this.InputDataLocation),
                    ex);
            }
        }

        /// <summary>
        /// Creates container specified by input parameter 'OutputDataLocation' if it does not 
        /// exist. Deletes all blobs from the container if it exists.
        /// </summary>
        private void ValidateOutputDataLocation()
        {
            try
            {
                CloudBlobContainer outputContainer = this.CloudClient.BlobClient.GetContainerReference(
                    this.OutputDataLocation);
                outputContainer.CreateIfNotExist();
                outputContainer.CleanIfExist();
            }
            catch (StorageException ex)
            {
                throw new ArgumentException(
                    string.Format(CultureInfo.InvariantCulture, "Error occurred while trying to initialize output location [{0}]", this.OutputDataLocation),
                    ex);
            }
        }
    }
}