﻿//------------------------------------------------------------------------------------------------- 
// <copyright file="OutlierDetectionWriter.cs" company="Microsoft">
// Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-------------------------------------------------------------------------------------------------

namespace Research.MapReduce.Samples.OutlierDetection
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using Microsoft.WindowsAzure.StorageClient;
    using Research.MapReduce.Core;

    /// <summary>
    /// Writes the key and value separated by a comma to a blob.
    /// </summary>
    /// <typeparam name="K">The type of key of record.</typeparam>
    /// <typeparam name="V">The type of value of record.</typeparam>
    [Serializable]
    public class OutlierDetectionWriter<K, V> : IRecordWriter<K, V>
    {
        /// <summary>
        /// Cloud blob where the results are stored.
        /// </summary>
        [NonSerialized]
        private CloudBlob blob;

        /// <summary>
        /// Custom delimiter for the key value pair.
        /// </summary>
        private string keyValueDelimiter = " ";

        /// <summary>
        /// Gets or sets the CloudClient.
        /// </summary>
        public CloudClient CloudClient { get; set; }

        /// <summary>
        /// Gets or sets Name of the container.
        /// </summary>
        public string ContainerName { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether to ignore the key while writing to the blob.
        /// </summary>
        public bool IgnoreKey { get; set; }

        /// <summary>
        /// Gets or sets delimiter to be used by the separate key and value.
        /// This is to help users to store Csv as values.
        /// </summary>
        public string KeyValueDelimiter
        {
            get { return this.keyValueDelimiter; }
            set { this.keyValueDelimiter = value; }
        }

        /// <summary>
        /// Gets or sets Column headers
        /// </summary>
        public string ColumnHeaders { get; set; }

        /// <summary>
        /// Write logic implementation.
        /// </summary>
        /// <param name="outputPartition">Name of the partition.</param>
        /// <param name="records">Collection of records.</param>
        public void Write(string outputPartition, IEnumerable<KeyValuePair<K, V>> records)
        {
            CloudBlobContainer container = CloudClient.BlobClient.GetContainerReference(this.ContainerName);
            container.CreateIfNotExist();
            this.blob = container.GetBlobReference(outputPartition);

            using (StreamWriter writer = new StreamWriter(this.blob.OpenWrite()))
            {
                if (!string.IsNullOrEmpty(this.ColumnHeaders))
                {
                    writer.Write(this.ColumnHeaders);
                    writer.WriteLine();
                }

                foreach (KeyValuePair<K, V> record in records)
                {
                    if (!this.IgnoreKey)
                    {
                        string keyString = record.Key as string;
                        if (string.IsNullOrEmpty(keyString))
                        {
                            writer.Write(record.Key);
                        }
                        else
                        {
                            writer.Write(keyString);
                        }

                        writer.Write(this.keyValueDelimiter);
                    }

                    string valueString = record.Value as string;

                    if (string.IsNullOrEmpty(valueString))
                    {
                        writer.Write(record.Value);
                    }
                    else
                    {
                        writer.Write(valueString);
                    }

                    writer.WriteLine();
                }
            }
        }

        /// <summary>
        /// Get the result.
        /// </summary>
        /// <returns>returns the results</returns>
        public IReduceResult<K, V> GetResult()
        {
            BlobResultCollection<K, V> result = new BlobResultCollection<K, V>
            {
                CloudClient = this.CloudClient,
                BlobUri = this.blob.Uri.AbsoluteUri,
                IgnoreKey = this.IgnoreKey,
                KeyValueDelimiter = this.keyValueDelimiter
            };

            return result;
        }
    }
}
