﻿//------------------------------------------------------------------------------------------------- 
// <copyright file="Program.cs" company="Microsoft">
// Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-------------------------------------------------------------------------------------------------

namespace Research.MapReduce.Samples.OutlierDetection
{
    using System;
    using System.Collections.Generic;
    using System.Configuration;
    using System.Globalization;
    using System.IO;
    using System.Linq;
    using Microsoft.WindowsAzure;
    using Microsoft.WindowsAzure.StorageClient;
    using Research.MapReduce.Core;
    using Research.MapReduce.Samples.OutlierDetection.Properties;

    class Program
    {
        #region Fields

        private const string PackageName = "OutlierDetectionSample";
        private static string masterConnectionString;
        private static CloudStorageAccount storageAccount;
        private static Dictionary<string, string> controllerArgs;

        #endregion

        #region Methods

        static void Main(string[] args)
        {
            // Read values of 'masterConnectionString' and 'storageAccount' from app.config.
            ValidateAndGetConfiguration();

            if (args.Length == 0)
            {
                Console.WriteLine(Environment.NewLine + @"Usage:- -InputDataLocation::<value> -OutputDataLocation::<value> -Mappers::<value> -Reducers::<value> -ColumnNames::<value> -NumberOfOutliers::<value> -JobTimeoutInMinutes::<value>" + Environment.NewLine);
                args = GetDefaultInputArgs();
                Console.WriteLine();
                Console.WriteLine("Using default arguments >> {0}" + Environment.NewLine, args.GetString());
            }

            // Populate controller parameter values from input arguments.
            controllerArgs = new Dictionary<string, string>();
            foreach (string arg in args)
            {
                string[] argArray = arg.Split(new string[] { "::" }, StringSplitOptions.RemoveEmptyEntries);
                controllerArgs.Add(argArray[0].TrimStart('-'), argArray[1]);
            }

            // Submit application for execution and wait till it completes.
            // Note:- Outlier detection is executed in two phase, using a timeout that 
            // is slightly greater than the outlierdetection job. Use a larger timeout 
            // if other applications have already been submitted before this one.
            int timeoutInMinutes = (int.Parse(controllerArgs[ParameterNames.JobTimeoutInMinutes], CultureInfo.InvariantCulture) * 2) + 2;
            SubmitAndWaitForApplication(timeoutInMinutes);

            Console.WriteLine("Press <ENTER> to exit.");
            Console.ReadLine();
        }

        /// <summary>
        /// Reads and validates values of 'masterConnectionString' and 'StorageConnectionString' 
        /// from app.config.         
        /// </summary>
        private static void ValidateAndGetConfiguration()
        {
            masterConnectionString = ConfigurationManager.AppSettings["MasterConnectionString"];
            string storageConnectionString = ConfigurationManager.AppSettings["MasterConnectionString"];

            if (string.IsNullOrEmpty(masterConnectionString) || string.IsNullOrWhiteSpace(masterConnectionString))
            {
                throw new InvalidOperationException(Resources.MasterConnectionStringConfigurationMissing);
            }

            if (string.IsNullOrEmpty(storageConnectionString) || string.IsNullOrWhiteSpace(storageConnectionString))
            {
                throw new InvalidOperationException(Resources.StorageConnectionStringConfigurationMissing);
            }

            if (!CloudStorageAccount.TryParse(masterConnectionString, out storageAccount))
            {
                throw new InvalidOperationException(Resources.MasterConnectionStringIncorrect);
            }

            if (!CloudStorageAccount.TryParse(storageConnectionString, out storageAccount))
            {
                throw new InvalidOperationException(Resources.StorageConnectionStringIncorrect);
            }
        }

        /// <summary>
        /// Gets default arguments for running the sample.
        /// </summary>
        /// <returns>String array containing the arguments for running the sample.</returns>
        private static string[] GetDefaultInputArgs()
        {
            string inputContainerName = "outlierdetection-input-" + Environment.UserName;
            string outputContainerName = "outlierdetection-output-" + Environment.UserName;

            string[] args = new string[7];

            // Arg0 - InputDataLocation
            CloudBlobContainer container = GetInitializedContainer(inputContainerName);
            CloudBlob blob = container.GetBlobReference("sampleinput.csv");
            blob.UploadText(Resources.sampleinput);
            args[0] = string.Format(CultureInfo.InvariantCulture, "-{0}::{1}", ParameterNames.InputDataLocation, blob.Uri);

            // Arg1 - OutputDataLocation          
            container = GetInitializedContainer(outputContainerName);
            args[1] = string.Format(CultureInfo.InvariantCulture, "-{0}::{1}", ParameterNames.OutputDataLocation, outputContainerName);

            // Arg2 - Mappers
            args[2] = string.Format(CultureInfo.InvariantCulture, "-{0}::{1}", ParameterNames.Mappers, 4);

            // Arg3 - Reducers
            args[3] = string.Format(CultureInfo.InvariantCulture, "-{0}::{1}", ParameterNames.Reducers, 4);

            // Arg4 - Reducers
            args[4] = string.Format(CultureInfo.InvariantCulture, "-{0}::{1}", ParameterNames.ColumnNames, "Location,Key");

            // Arg5 - Reducers
            args[5] = string.Format(CultureInfo.InvariantCulture, "-{0}::{1}", ParameterNames.NumberOfOutliers, 10);

            // Arg6 - JobTimeoutInMinutes
            args[6] = string.Format(CultureInfo.InvariantCulture, "-{0}::{1}", ParameterNames.JobTimeoutInMinutes, 2);

            return args;
        }

        /// <summary>
        /// Creates a CloudBlobContainer if it does not already exist. Cleans the container if it 
        /// exists.
        /// </summary>
        /// <param name="name">Name of the container to be initialized.</param>
        /// <returns>CloudBlobContainer object.</returns>
        private static CloudBlobContainer GetInitializedContainer(string name)
        {
            CloudBlobContainer container = storageAccount.CreateCloudBlobClient().
                GetContainerReference(name);
            container.CreateIfNotExist();
            container.CleanIfExist();
            return container;
        }

        /// <summary>
        /// Submits application for execution and waits till it completes.
        /// </summary>
        /// <param name="timeoutInMinutes">Timeout value for the application in minutes</param>
        private static void SubmitAndWaitForApplication(int timeoutInMinutes)
        {
            MapReduceClient client = new MapReduceClient(PackageName, Guid.NewGuid().ToString("N"), typeof(OutlierDetectionController).AssemblyQualifiedName, controllerArgs);
            client.Submit(masterConnectionString);

            Console.WriteLine("Application submitted successfully...");

            Console.WriteLine("Waiting for completion...");

            bool concluded = client.WaitForCompletion(masterConnectionString, TimeSpan.FromMinutes(timeoutInMinutes), TimeSpan.FromSeconds(5));
            if (concluded)
            {
                if (client.Succeeded)
                {
                    Console.WriteLine("Application completed successfully." + Environment.NewLine);

                    Console.WriteLine("Outliers are::" + Environment.NewLine);
                    string finalClustersBlobUri = client.Results.Where(tuple => tuple.Key == ResultNames.OutputContainer).First().Value;
                    CloudBlob blob = (CloudBlob)storageAccount.CreateCloudBlobClient().GetContainerReference(finalClustersBlobUri).ListBlobs().First();
                    using (StreamReader sr = new StreamReader(blob.OpenRead()))
                    {
                        Console.WriteLine(sr.ReadToEnd());
                    }
                }
                else
                {
                    Console.WriteLine("Application failed due to: " + Environment.NewLine + "{0}", client.FailReason);
                }
            }
            else
            {
                Console.WriteLine("Application wait timeout elapsed.");
            }
        }

        #endregion
    }
}